/*
 ** Copyright(C) 2006 INL
 **     written by Eric Leblond <regit@inl.fr>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, version 2 of the License.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "conffile.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "callbacks.h"


/**
 * Taken a conf file and hash containing options, fill the hash with options values.
 * 
 * \param filename File name
 * \param array_size Size of the array
 * \param symbols (Pointer to) Hash containing options
 * \return 1 if OK, 0 otherwise
 */

int parse_conffile(char * filename,gint array_size,confparams* symbols) 
{
	GScanner*  scanner;
	GTokenType dnentry=G_TOKEN_NONE;
	int fd,i;
	gboolean done;
	confparams* current_symbol=NULL;

	scanner=g_scanner_new(NULL);
	fd=open(filename,O_RDONLY);
	if (fd == -1)
    {
        return 0;
    }
	g_scanner_input_file(scanner,fd);
	for (i = 0; i < array_size; i++)
		g_scanner_scope_add_symbol (scanner, 0, symbols[i].name, GINT_TO_POINTER (i));

	done = FALSE;
	while (!done){
		dnentry=g_scanner_get_next_token (scanner);
		if (dnentry == G_TOKEN_EOF){
			done=TRUE;
			break;
		}
		if (dnentry == G_TOKEN_SYMBOL){
			current_symbol=NULL;
			for (i = 0; i < array_size; i++){
				if (i == GPOINTER_TO_INT(scanner->value.v_symbol)){
					current_symbol=symbols+i;
					break;
				}
			}
			if (current_symbol != NULL){
				dnentry=g_scanner_get_next_token (scanner); 
				if (dnentry ==  G_TOKEN_EQUAL_SIGN){ 
					dnentry=g_scanner_get_next_token (scanner); 
					switch (dnentry){
						case G_TOKEN_STRING :
							/* test if element want a string */
							if (current_symbol->value_type == G_TOKEN_STRING){
                                /* free allocated value */
                                g_free(current_symbol->v_char);
                                /* put new value */
								current_symbol->v_char=g_strdup(scanner->value.v_string);
							} else {
								g_warning("Bad argument value for %s at %u",
										current_symbol->name,scanner->line);
								g_scanner_destroy (scanner);
								return 0;
							}
							break;
						case G_TOKEN_INT :
							/* test if element want a string */
							if (current_symbol->value_type == G_TOKEN_INT){
								current_symbol->v_int=scanner->value.v_int;
							} else {
								g_warning("Bad argument value for %s at %u",
										current_symbol->name,scanner->line);
								g_scanner_destroy (scanner);
								return 0;
							}
							break;
						default :
							g_warning("Bad argument !\n");
					}
				}
			} else {
				g_warning("Did not find a symbol at %d,%d",scanner->line,scanner->position);
			}
		}
	}
	g_scanner_destroy (scanner);
	close(fd);
	return 1;
}


/**
 * Fetch value of an option and return a pointer to it.
 * 
 * \param symbols Option array 
 * \param array_size Number of entries in symbols
 * \param confparam String identifier of parameter to get
 * \return Pointer to parameter value
 */
gpointer get_confvar_value(confparams* symbols,gint array_size,gchar * confparam){
	gpointer value=NULL;
	int i;
	int token_type;
	/* go through symbol table */
	for (i = 0; i < array_size; i++){
		if (! strcmp(symbols[i].name,confparam) ){
			token_type = symbols[i].value_type;
			switch ( token_type ){
				case G_TOKEN_STRING :
					value=g_strdup(symbols[i].v_char);
					break;
				case G_TOKEN_INT :
					value=&(symbols[i].v_int);
					break;
				default :
					value=NULL;
					break;
			} 
			return value;
		}
	}
	return NULL;
}

int free_confparams(confparams symbols[],gint array_size)
{
  int i;
	for (i = 0; i < array_size; i++){
        if (symbols[i].value_type == G_TOKEN_STRING){
            g_free(symbols[i].v_char);
        }
    }
    return 0;
}

#define CONFILE ".nuapprc"

gchar *get_config_file()
{
  gchar *configfile=NULL;
  gchar *home = getenv("HOME");
  if (home == NULL)
      return NULL;            
  if (strlen(home)<512){
      configfile=g_new0(gchar,strlen(home)+strlen(CONFILE)+2);
  }
  if (configfile){
      snprintf(configfile,strlen(home)+strlen(CONFILE)+2,"%s/%s", home,CONFILE);
  }
  return configfile;
}

int parse_nuapp_params(gchar* configfile)
{
  gpointer vpointer;
  confparams nuapp_vars[] = {
      { "host" , G_TOKEN_STRING, 0 , g_strdup("localhost") },
      { "port" ,G_TOKEN_INT , 4130,NULL },
      { "username" , G_TOKEN_STRING , 0 ,g_strdup("")},
      { "auth_at_start" ,G_TOKEN_INT , 0,NULL },
      { "start_minimized" ,G_TOKEN_INT , 0,NULL },
      { "interval" , G_TOKEN_INT , 100 , NULL }
  };

/* parse conf file */
  parse_conffile(configfile,sizeof(nuapp_vars)/sizeof(confparams),nuapp_vars);
  /* set variables */
  vpointer=get_confvar_value(nuapp_vars,sizeof(nuapp_vars)/sizeof(confparams),"host");
  nuapp_datas.host=(char *)vpointer;
  vpointer=get_confvar_value(nuapp_vars,sizeof(nuapp_vars)/sizeof(confparams),"port");
  nuapp_datas.port=*(int *)vpointer;
  vpointer=get_confvar_value(nuapp_vars,sizeof(nuapp_vars)/sizeof(confparams),"username");
  nuapp_datas.username=(char *)vpointer;
  vpointer=get_confvar_value(nuapp_vars,sizeof(nuapp_vars)/sizeof(confparams),"auth_at_start");
  nuapp_datas.auth_at_start=*(int *)vpointer;
  vpointer=get_confvar_value(nuapp_vars,sizeof(nuapp_vars)/sizeof(confparams),"start_minimized");
  nuapp_datas.start_minimized=*(int *)vpointer;
  vpointer=get_confvar_value(nuapp_vars,sizeof(nuapp_vars)/sizeof(confparams),"interval");
  nuapp_datas.interval=*(int *)vpointer;

}

int dump_nuauth_params_to_file(gchar *filename)
{
  FILE* configfile = fopen(filename,"w");

  if (configfile==NULL){
    return 0;
  }
  fprintf(configfile,"%s = \"%s\"\n","host",  nuapp_datas.host);
  fprintf(configfile,"%s = %d\n","port",  nuapp_datas.port);
  fprintf(configfile,"%s = \"%s\"\n","username",  nuapp_datas.username);
  fprintf(configfile,"%s = %d\n","interval",  nuapp_datas.interval);
  fprintf(configfile,"%s = %d\n","auth_at_start",  nuapp_datas.auth_at_start);
  fprintf(configfile,"%s = %d\n","start_minimized",  nuapp_datas.start_minimized);

  fclose(configfile);
  return 1;
}
